"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const card_1 = __importDefault(require("../card"));
const card_constant_1 = require("../card/card.constant");
/**
 * Represents a deck of playing cards.
 */
class Deck {
    /**
     * Creates a new instance of the Deck class.
     * @param deck - An optional array of cards to initialize the deck with.
     */
    constructor(deck) {
        this.deck = [];
        this.drawCards = [];
        if (deck) {
            this.deck = deck;
        }
        else {
            this.initDeck().getDeck();
        }
    }
    /**
     * Retrieves the deck of cards.
     * @returns {Card[]} The deck of cards.
     */
    getDeck() {
        return this.deck;
    }
    /**
     * Initializes the deck with a standard set of playing cards.
     * @returns The current instance of the Deck class.
     */
    initDeck() {
        const deck = [];
        for (let i = 0; i < card_constant_1.Suits.length; i++) {
            for (let x = 0; x < card_constant_1.Values.length; x++) {
                const card = new card_1.default(card_constant_1.Suits[i], card_constant_1.Values[x]);
                deck.push(card);
            }
        }
        this.deck = deck;
        return this;
    }
    /**
     * Resets the deck by reinitializing it with a standard set of playing cards.
     * @returns The current instance of the Deck class.
     */
    resetDeck() {
        this.initDeck();
        return this;
    }
    /**
     * Shuffles the deck by swapping random cards multiple times.
     * @param nbShuffleTime The number of times to shuffle the deck. Default is 1000.
     * @returns The updated Deck instance with the shuffled deck.
     */
    shuffleDeck(nbShuffleTime = 1000) {
        const deck = this.deck.slice();
        for (let i = 0; i < nbShuffleTime; i++) {
            const pos1 = Math.floor((Math.random() * deck.length));
            const pos2 = Math.floor((Math.random() * deck.length));
            const tmp = deck[pos1];
            deck[pos1] = deck[pos2];
            deck[pos2] = tmp;
        }
        this.deck = deck;
        return this;
    }
    /**
     * Draws a card from the deck and adds it to the drawCards array.
     * @returns {Card | undefined} The card that was drawn from the deck, or undefined if the deck is empty.
     */
    drawCard() {
        const card = this.deck.shift();
        if (card) {
            this.drawCards.push(card);
        }
        return card;
    }
}
exports.default = Deck;
